#include <ace/OS.h>
#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <ace/streams.h>

#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "ObjectAdapter.h"

ObjectAdapter::ObjectAdapter (CORBA::ORB_var orbVar,
                              const CORBA::Boolean agent)
  : orbVar_ (orbVar), agent_ (agent), pParentOA_ (0),
    rootPoaVar_ (0), thisPoaVar_ (0), poaManagerVar_ (0), managerIdVar_ (0)
{
  initialiseRootPOA ();
  ACE_OS::strcpy (poaName_, ACE_TEXT ("thisPOA"));
}

ObjectAdapter::ObjectAdapter (CORBA::ORB_var orbVar,
                              const char *poaName,
                              const CORBA::Boolean agent)
  : orbVar_ (orbVar), agent_ (agent), pParentOA_ (0),
    rootPoaVar_ (0), thisPoaVar_ (0), poaManagerVar_ (0), managerIdVar_ (0)
{
  initialiseRootPOA ();
  ACE_OS::strcpy (poaName_, poaName);
}

ObjectAdapter::ObjectAdapter (CORBA::ORB_var orbVar,
                              ObjectAdapter *pAdapter,
                              const char *poaName,
                              const CORBA::Boolean persistent,
                              const CORBA::Boolean agent)
  : orbVar_ (orbVar), agent_ (agent), pParentOA_ (pAdapter),
    rootPoaVar_ (0), thisPoaVar_ (0), poaManagerVar_ (0), managerIdVar_ (0)
{
  initialiseRootPOA ();
  ACE_OS::strcpy (poaName_, poaName);
  create (persistent);
}

void
ObjectAdapter::initialiseRootPOA ()
{
   CORBA::Object_var objVar = orbVar_->resolve_initial_references (ACE_TEXT ("RootPOA"));
   rootPoaVar_ = PortableServer::POA::_narrow (objVar.in ());
   poaManagerVar_ = rootPoaVar_->the_POAManager ();
   poaManagerVar_->activate ();
}

void
ObjectAdapter::createTransient (PortableServer::ServantBase *pImpl)
{
  if (pImpl == 0)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Null implementation passed to createTransient()\n")));
      exit (1);
    }

  // Create a transient POA.
  create ((CORBA::Boolean) 0);

  // Activate the transient object reference.
  CORBA::String_var serverNameVar = ACE_OS::strdup (ACE_TEXT ("thisServer"));
  addServantToAdapter (pImpl, serverNameVar.in (), 0);
}

void
ObjectAdapter::createPersistent (PortableServer::ServantBase *pImpl)
{
  if (pImpl == 0)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Null implementation passed to createPersistent()\n")));
      exit (1);
    }

  // Create a persistent POA.
  create ((CORBA::Boolean) 1);

  // Activate the persistent object reference.
  CORBA::String_var serverNameVar = ACE_OS::strdup (ACE_TEXT ("thisServer"));
  addServantToAdapter (pImpl, serverNameVar.in (), 1);
}

void
ObjectAdapter::create (CORBA::Boolean persistent)
{
  // Create policies.
  CORBA::PolicyList policies;
  if (persistent)
    {
      policies.length (3);
      policies[(CORBA::ULong)0] = rootPoaVar_->create_lifespan_policy (PortableServer::PERSISTENT);
      policies[(CORBA::ULong)1] = rootPoaVar_->create_id_assignment_policy (PortableServer::USER_ID);
      policies[(CORBA::ULong)2] = rootPoaVar_->create_implicit_activation_policy (PortableServer::NO_IMPLICIT_ACTIVATION);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: created persistent policies\n")));
    }
  else
    {
      policies.length (1);
      policies[(CORBA::ULong)0] = rootPoaVar_->create_lifespan_policy (PortableServer::TRANSIENT);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: created transient policy\n")));
    }

  // Create a child POA with the right policies.
  CORBA::String_var poaNameVar = ACE_OS::strdup (poaName_);
  ACE_TRY_NEW_ENV
    {
      thisPoaVar_ = rootPoaVar_->create_POA (poaNameVar.in (), poaManagerVar_.in (), policies ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: failed to create POA\n"));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: failed to create POA\n")));
    }
  ACE_ENDTRY;
}

void
ObjectAdapter::createTransient ()
{
  create ((CORBA::Boolean) 0);
}

void
ObjectAdapter::createPersistent ()
{
  create ((CORBA::Boolean) 1);
}

PortableServer::POA_var
ObjectAdapter::getPOA ()
{
  return thisPoaVar_;
}

ObjectAdapter *
ObjectAdapter::getParent () const
{
  return pParentOA_;
}

char *
ObjectAdapter::getPoaName ()
{
  return (char*) poaName_;
}

void
ObjectAdapter::addServantToAdapter (PortableServer::ServantBase *pImpl,
                                    const char *name,
                                    CORBA::Boolean persistent)
{
  CORBA::String_var servantNameVar = ACE_OS::strdup (name);

  if (persistent == 1)
    {
      // Create an ID for the persistent object, and activate it.
      PortableServer::ObjectId_var serverIdVar = PortableServer::string_to_ObjectId (servantNameVar.in ());
      ACE_TRY_NEW_ENV
        {
          getPOA ()->activate_object_with_id (serverIdVar.in (), pImpl ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to activate persistent servant\n"));
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to activate persistent servant\n")));
        }
      ACE_ENDTRY;
    }
  else
    {
      // Allow the system to create an ID for the transient object, and activate it.
      ACE_TRY_NEW_ENV
        {
          getPOA ()->activate_object (pImpl ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to activate transient servant\n"));
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to activate transient servant\n")));
        }
      ACE_ENDTRY;
    }
}

void
ObjectAdapter::activateManager ()
{
   thisPoaVar_->the_POAManager ()->activate ();
}
