/*
 * Basic log service example.
 */

#include "BasicLogExample.h"


#define LOG_EVENTS 4

BasicLogExample::BasicLogExample (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar), basicLogFactoryVar_ (0)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: ORB not initialised\n")));
      exit (-1);
    }

  // Resolve to the basic log factory.
  CORBA::Object_var objVar = 0;
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to basic log factory\n")));
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("BasicLogFactory") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve to basic log factory\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to resolve to basic log factory\n")));
      exit (-1);
    }
  ACE_ENDTRY;

  basicLogFactoryVar_ = DsLogAdmin::BasicLogFactory::_narrow (objVar.in ());
  if (CORBA::is_nil (basicLogFactoryVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null basic log factory\n")));
      exit (-1);
    }
}

BasicLogExample::~BasicLogExample ()
{
}

DsLogAdmin::BasicLog_ptr
BasicLogExample::createLog ()
{
  DsLogAdmin::BasicLog_var basicLogVar;
  DsLogAdmin::LogFullActionType logFullAction = DsLogAdmin::wrap;
  CORBA::ULongLong max_size = 0; // 0 means "infinite"
  DsLogAdmin::LogId logId = 0;

  basicLogVar = basicLogFactoryVar_->create (logFullAction, max_size, logId);

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Created log with ID: %i\n"), logId));
  return basicLogVar._retn ();
}

void
BasicLogExample::writeLog (DsLogAdmin::BasicLog_ptr log)
{
  DsLogAdmin::Anys data (LOG_EVENTS);
  data.length (LOG_EVENTS);

  double d = 42.424242;
  int n = 123456789;
  char a = 'a';

  data[0] <<= ACE_OS::strdup (ACE_TEXT ("Hello world"));
  data[1] <<= d;
  data[2] <<= n;
  data[3] <<= a;

  ACE_TRY_NEW_ENV
    {
      log->write_records (data ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
 ACE_CATCH (DsLogAdmin::LogFull, logFull)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Log is full\n")));
      exit (1);
    }
 ACE_CATCH (DsLogAdmin::LogLocked, logLocked)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Log is locked\n")));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to write to log\n"));
      exit (1);
    }
  ACE_ENDTRY;
}

void
BasicLogExample::queryLog (DsLogAdmin::BasicLog_ptr log)
{
  const char *grammar = ACE_TEXT ("EXTENDED_TCL");
  const char *constraint = ACE_TEXT ("$.info == 'Hello world'");
  DsLogAdmin::Iterator_var iterVar;
  DsLogAdmin::RecordList_var recordsVar;

  ACE_TRY_NEW_ENV
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Querying using constraint: %s\n"), constraint));
      recordsVar = log->query (grammar, constraint, iterVar ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      if (recordsVar->length () == 1)
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("As expected:\n")));
        }
      else
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Expected 1:\n")));
        }

      ACE_DEBUG ((LM_INFO, ACE_TEXT ("-------------------------\n")));

      for (unsigned int i = 0; i < recordsVar->length (); i++)
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("The ID of the LogRecord matching the constraint <%s> is: %i\n"), constraint, (CORBA::ULong)(recordsVar[i].id)));
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("-------------------------\n")));
        }
    }
  ACE_CATCH (DsLogAdmin::InvalidGrammar, ig)
    {
      ACE_PRINT_EXCEPTION (ig, ACE_TEXT ("ERROR: Failed to query log: invalid grammar\n"));
      exit (1);
    }
  ACE_CATCH (DsLogAdmin::InvalidConstraint, ic)
    {
      ACE_PRINT_EXCEPTION (ic, ACE_TEXT ("ERROR: Failed to query log: invalid constraint\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to query log\n"));
      exit (1);
    }
  ACE_ENDTRY;
}

int main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  BasicLogExample *pExample;
  DsLogAdmin::BasicLog_var logVar;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Initialising ORB\n")));
  orbVar = CORBA::ORB_init (argc, argv);

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Running example\n")));
  pExample = new BasicLogExample (orbVar);
  logVar = pExample->createLog ();
  pExample->writeLog (logVar.in ());
  pExample->queryLog (logVar.in ());

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Destroying log\n")));
  logVar->destroy ();

  delete pExample;
  return 0;
}
