/*
 * Client for the naming service bank example.
 */

#include "Naming/Bank/BankS.h"
#include "Naming/Bank/BankC.h"
#include "Naming/Bank/Util.h"

#include <ace/OS.h>
#include <tao/corba.h>


namespace
{
  const char* names[10] =
    {
      "George",
      "Lenon",
      "Stephen",
      "Mary",
      "Gary",
      "Jonathan",
      "Luke",
      "Rachel",
      "Paul",
      "Steve"
    };
};


static void showAccount (const Bank::Account_ptr account)
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Current Balance for Account Holder %s is: %.2f\n"), account->name (), account->balance ()));
}

int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  Util *pUtil;
  Bank::AccountManager_var accMgrVar;
  
  // Get the ORB reference.
  orbVar = CORBA::ORB_init (argc, argv);
  if (CORBA::is_nil (orbVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not instantiate the ORB\n")));
      exit (-1);
    }

  // Get the account manager reference.
  pUtil = new Util (orbVar);
  accMgrVar = pUtil->getAccManagerObject ();
  if (CORBA::is_nil (accMgrVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not get the account manager reference\n")));
      exit (-1);
    }

  // Create the accounts.    
  for (CORBA::ULong i = 0; i < 10; i++)
    {
      ACE_TRY_NEW_ENV
        {
          accMgrVar->createAccount (names[i], 10000 ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCH (Bank::AccountManager::AlreadyExists, ae)
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Account for %s already exists: using the existing account information\n"), names[i]));
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to create the accounts\n"));
          exit (-1);
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to create the accounts\n")));
          exit (-1);
        }
      ACE_ENDTRY;
    }

  // Run the example.
  ACE_TRY_NEW_ENV
    {
      Bank::Account_var georgeAccVar = accMgrVar->getAccount (ACE_TEXT ("George") ACE_ENV_ARG_PARAMETER);
      showAccount (georgeAccVar.in () ACE_ENV_ARG_PARAMETER);

      georgeAccVar->deposit (21000 ACE_ENV_ARG_PARAMETER);
      showAccount (georgeAccVar.in () ACE_ENV_ARG_PARAMETER);

      georgeAccVar->withdraw (9000 ACE_ENV_ARG_PARAMETER);
      showAccount (georgeAccVar.in () ACE_ENV_ARG_PARAMETER);

      georgeAccVar->withdraw (3900 ACE_ENV_ARG_PARAMETER);
      showAccount (georgeAccVar.in () ACE_ENV_ARG_PARAMETER);

      Bank::Account_var rachelAccVar = accMgrVar->getAccount (ACE_TEXT ("Rachel") ACE_ENV_ARG_PARAMETER);
      showAccount (rachelAccVar.in () ACE_ENV_ARG_PARAMETER);
      
      ACE_TRY_CHECK;
    }
  ACE_CATCH (Bank::AccountManager::NotFound, nf)
    {
      ACE_PRINT_EXCEPTION (nf, ACE_TEXT ("ERROR: Account not found\n"));
      exit (1);
    }
  ACE_CATCH (Bank::Account::InsufficientFunds, ifunds)
    {
      ACE_PRINT_EXCEPTION (ifunds, ACE_TEXT ("ERROR: You cannot withdraw more money than you already have\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Unexpected exception\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Unexpected exception\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Complete.
  orbVar->destroy ();
  delete pUtil;
  return 0;
}
