/*
 * A utility to administrate a mail server, allowing user accounts to be created and
 * deleted.
 */

#include "Admin.h"


/*
 * @func Constructs an Admin object
 * @parm | orbVar | a reference to the ORB
 * @rdesc LM_ERROR message and exit on error
 */
Admin::Admin (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar), mailServerVar_ (0)
{
  ChannelUtil *pUtil = new ChannelUtil (orbVar_);

  CORBA::Object_var objVar = pUtil->readIOR (ACE_TEXT ("MailServer.ior"));
  if (CORBA::is_nil (objVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("\nCould not read the Mail Server's IOR\nPossible causes:\n")));
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("(1) the Mail Server has not been started\n")));
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("(2) the Mail Server was started but has been stopped\n")));
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("(3) the Mail Server was started in a different directory\n")));
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("(4) the Mail Server IOR file (MailServer.ior) has been deleted or corrupted\n")));
      exit (-1);
    }
  mailServerVar_ = Mail::MailServer::_narrow (objVar.in ());
  if (CORBA::is_nil (mailServerVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to narrow the mail server reference\n")));
      exit (-1);
    }

  delete pUtil;
}

/*
 * @func Destroys this object
 */
Admin::~Admin ()
{
}

/*
 * @func Creates an account for the specified user
 * @parm | username | the user's name
 * @rdesc LM_ERROR message if an account exists for this user
 */
void
Admin::createAccount (char *username)
{
  ACE_TRY_NEW_ENV
    {
      mailServerVar_->createAccount (username ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (Mail::AccountExists, ae)
    {
      ACE_PRINT_EXCEPTION (ae, ACE_TEXT ("Account already exists\n"));
    }
  ACE_ENDTRY;
}

/*
 * @func Deletes an account for the specified user
 * @parm | username | the user's name
 * @rdesc LM_ERROR message if no account exists for this user
 */
void
Admin::deleteAccount (char *username)
{
  ACE_TRY_NEW_ENV
    {
      mailServerVar_->deleteAccount (username ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (Mail::NoSuchAccount, nsa)
    {
      ACE_PRINT_EXCEPTION (nsa, ACE_TEXT ("Account does not exist\n"));
    }
  ACE_ENDTRY;
}

int
main (int argc, char *argv[])
{
  CORBA::ORB_var orbVar;
  Admin *pAdmin;
  char *command;
  char *username;

  if (argc < 3)
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Usage: admin [create|destroy] [username]\n")));
      exit (-1);
    }

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Initialising ORB\n")));
  orbVar = CORBA::ORB_init (argc, argv);

  pAdmin = new Admin (orbVar);
  command = argv[1];
  username = argv[2];
  if (ACE_OS::strcmp (command, ACE_TEXT ("create")) == 0)
    {
      pAdmin->createAccount (username);
    }
  else if (ACE_OS::strcmp (command, ACE_TEXT ("delete")) == 0)
    {
      pAdmin->deleteAccount (username);
    }
  else
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Unrecognised command: %s\n"), command));
    }

  delete pAdmin;

  return 0;
}
