/*
 * A simple mail tool with a text based interface that can be used to send
 * and receive mail messages.
 */

#include "MailTool.h"


/*
 * @func Constructs a MailTool object
 * @parm | orbVar | reference to the ORB
 * @parm | username | the user's name
 * @rdesc NoSuchAccount is thrown if an account does not exist for the user
 */
MailTool::MailTool (CORBA::ORB_var orbVar, char *username)
  : mailBoxVar_ (0)
{
  CORBA::Object_var objVar;
  char iorFileName[64];
  ChannelUtil *pUtil = new ChannelUtil (orbVar);

  ACE_OS::strcpy (iorFileName, username);
  ACE_OS::strcat (iorFileName, ACE_TEXT (".ior"));
  if (! pUtil->fileExists (iorFileName))
    {
      ACE_THROW (Mail::NoSuchAccount ());
    }

  objVar = pUtil->readIOR (iorFileName);
  mailBoxVar_ = Mail::MailBox::_narrow (objVar.in ());

  delete pUtil;
}

/*
 * @func Destroys this object
 */
MailTool::~MailTool ()
{
}

/*
 * @func Sends a message to the mailbox
 * @parm message | the message to send
 * @parm to | string sequence of recipients
 * @rdesc LM_ERROR message on error
 */
void
MailTool::sendMessage (char *message, Mail::StringSeq_var to)
{
  ACE_TRY_NEW_ENV
    {
      mailBoxVar_->sendMessage (message, to.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to send message\n"));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to send message\n")));
    }
  ACE_ENDTRY;
}

/*
 * @func Receives a message from the mailbox
 * @parm sender | string to hold the sender information
 * @parm message | string to hold the message information
 * @rdesc boolean indicating if a message was waiting: if true, sender and message
 *        will contain the information
 */
CORBA::Boolean
MailTool::receiveMessage (CORBA::String_var &sender, CORBA::String_var &message)
{
  CORBA::Boolean b = mailBoxVar_->receiveMessage (sender.out (), message.out ());
  return b;
}

/*
 * @func Utility method: converts a comma-separated list of names to a string sequence
 * @parm list | string with the comma-separated names
 * @rdesc string sequence of the individual names
 */
Mail::StringSeq *
parseList (char *list, const char *separators)
{
  Mail::StringSeq_var toVar = new Mail::StringSeq;

  char *token;
  token = ACE_OS::strtok (list, separators);
  int count = 0;
  while (token != 0)
    {
      // Get the next token.
      char *item = ACE_OS::strdup (token);

      // Add it to the sequence.
      toVar->length (++count);
      (*toVar)[count-1] = CORBA::string_dup (item);

      // Repeat.
      token = ACE_OS::strtok (0, separators);
    }

  return toVar._retn ();
}

/*
 * @func Utility method: get a line of input from the console, without a trailing carriage return / linefeed
 * @parm buffer | caller-allocated buffer for the input
 * @parm size | the size of the buffer
 */
void getInput (char *buffer, int size)
{
  ACE_OS::fgets (buffer, size, stdin);
  buffer[ACE_OS::strcspn (buffer, "\n")] = '\0';
}

int main (int argc, char *argv[])
{
  char *username;
  MailTool *pMailTool;

  if (argc < 2)
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Usage: mailtool [username]\n")));
      exit (-1);
    }
  username = argv[1];

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Initialising ORB\n")));
  CORBA::ORB_var orbVar = CORBA::ORB_init (argc, argv);

  ACE_TRY_NEW_ENV
    {
      pMailTool = new MailTool (orbVar.in (), username ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (Mail::NoSuchAccount, nsa)
    {
      ACE_DEBUG ((LM_ERROR, "User %s does not exist\n", username));
      ACE_DEBUG ((LM_ERROR, "Please use the admin tool to create users"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to initialise the mail tool\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to initialise the mail tool\n")));
      exit (1);
    }
  ACE_ENDTRY;

  const int BUFFER_SIZE = 256;
  char line[BUFFER_SIZE];
  char recipients[BUFFER_SIZE];
  char message[BUFFER_SIZE];
  CORBA::Boolean keepGoing = 1;

  while (keepGoing)
    {
      // Display options and get the user's selection.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("(S)end, (R)eceive or (Q)uit?\n")));
      getInput (line, BUFFER_SIZE);

      if ((ACE_OS::strcmp (line, ACE_TEXT ("send")) == 0) || (ACE_OS::strcmp (line, ACE_TEXT ("s")) == 0))
        {
          // Send a message.
        
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Please enter the recipients of this mail message. " \
                                         "You may have more than one recipient as long as they " \
                                         "are seperated by the ',' character\n")));
          getInput (recipients, BUFFER_SIZE);
          Mail::StringSeq_var toVar = parseList (recipients, ACE_TEXT (","));

          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Please enter the message to send\n")));
          getInput (message, BUFFER_SIZE);
          pMailTool->sendMessage (message, toVar);
        }
      else if ((ACE_OS::strcmp (line, ACE_TEXT ("receive")) == 0) || (ACE_OS::strcmp (line, ACE_TEXT ("r")) == 0))
        {
          // Try to receive a message.

          CORBA::String_var senderVar;
          CORBA::String_var messageVar;

          if (pMailTool->receiveMessage (senderVar, messageVar))
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("Received mail from %s\n"), senderVar.in ()));
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("Message: %s\n"), messageVar.in ()));
            }
          else
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("No new messages\n")));
            }
        }
      else if ((ACE_OS::strcmp (line, ACE_TEXT ("quit")) == 0) || (ACE_OS::strcmp (line, ACE_TEXT ("q")) == 0))
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Exiting\n")));
          keepGoing = 0;
        }
      else
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Unrecognised command: %s\n"), line));
        }
    }

  delete pMailTool;

  exit (0);
}
