/*
 * Creates structured events representing stock quotes and pushes them to
 * an event channel.
 */

#include "StockQuote.h"


namespace
{
  // All the companies about whom stock quotes will be generated.
  const int MAX_COMPANIES = 25;
  const char * const companies[MAX_COMPANIES] =
    {
      ACE_TEXT ("apple"),
      ACE_TEXT ("boeing"),
      ACE_TEXT ("coca-cola"),
      ACE_TEXT ("disney"),
      ACE_TEXT ("eds"),
      ACE_TEXT ("ford"),
      ACE_TEXT ("geico"),
      ACE_TEXT ("honeywell"),
      ACE_TEXT ("ibm"),
      ACE_TEXT ("j. p. morgan"),
      ACE_TEXT ("kellogs"),
      ACE_TEXT ("levi strauss"),
      ACE_TEXT ("microsoft"),
      ACE_TEXT ("nike"),
      ACE_TEXT ("oracle"),
      ACE_TEXT ("phillips"),
      ACE_TEXT ("quantum"),
      ACE_TEXT ("reebok"),
      ACE_TEXT ("sun microsystems"),
      ACE_TEXT ("texas instruments"),
      ACE_TEXT ("unisys"),
      ACE_TEXT ("viacom"),
      ACE_TEXT ("wells fargo"),
      ACE_TEXT ("xerox"),
      ACE_TEXT ("york international")
    };

  // A record of  the last quote for each company.
  short quotes[MAX_COMPANIES] =
    {
      100, 100, 100, 100, 100,
      100, 100, 100, 100, 100,
      100, 100, 100, 100, 100,
      100, 100, 100, 100, 100,
      100, 100, 100, 100, 100
    };
}

StockQuote::~StockQuote ()
{
}

CosNotification::StructuredEvent
StockQuote::createNextEvent ()
{
  CosNotification::StructuredEvent event;

  // Event header.
  short priority = (short) (rand () % 10);
  CosNotification::EventType type;
  type.domain_name = ACE_OS::strdup (ACE_TEXT ("News"));
  type.type_name = ACE_OS::strdup (ACE_TEXT ("StockQuote"));
  CosNotification::FixedEventHeader fixedHeader;
  fixedHeader.event_type = type;
  fixedHeader.event_name = ACE_OS::strdup (ACE_TEXT ("message"));
  CosNotification::PropertySeq varHeader;
  varHeader.length (2);
  varHeader[0].name = ACE_OS::strdup (CosNotification::Timeout);
  varHeader[0].value <<= (CORBA::ULongLong) 10 * 100 * 1000 * 60 * 5;
  varHeader[1].name = ACE_OS::strdup (CosNotification::Priority);
  varHeader[1].value <<= priority;
  event.header.fixed_header = fixedHeader;
  event.header.variable_header = varHeader;

  // Filterable data.
  int companyIndex = rand () % MAX_COMPANIES;
  const char *company = companies[companyIndex];
  CosNotification::PropertySeq filterable;
  filterable.length (3);
  filterable[0].name = ACE_OS::strdup (ACE_TEXT ("bureau"));
  filterable[0].value <<= ACE_OS::strdup (ACE_TEXT ("Wall Street"));
  filterable[1].name = ACE_OS::strdup (ACE_TEXT ("company"));
  filterable[1].value <<= ACE_OS::strdup (company);
  filterable[2].name = ACE_OS::strdup (CosNotification::Priority);
  filterable[2].value <<= priority;
  event.filterable_data = filterable;
  
  // Remainder-of-body
  char *news = generateNextMessage (companyIndex);
  event.remainder_of_body <<= ACE_OS::strdup (news);

  return event;
}

/*
 * @func Randomly generate a new stock quote for a given company
 * @parm | companyIndex | the index of the company for which to generate the new quote
 * @rdesc the new stock quote
 */
char *
StockQuote::generateNextMessage (int companyIndex)
{
  const char *company = companies[companyIndex];
  short lastValue = quotes[companyIndex];

  // Randomly generate a new value and store it for future reference.
  short nextValue = (short) rand () % 500;
  quotes[companyIndex] = nextValue;

  // Construct the message.

  char lastStr[4];
  ACE_OS::sprintf (lastStr, "%i", lastValue);
  char nextStr[4];
  ACE_OS::sprintf (nextStr, "%i", nextValue);

  char message[60];
  ACE_OS::strcpy (message, company);
  if (nextValue < lastValue)
    {
      ACE_OS::strcat (message, ACE_TEXT (" dropped from "));
    }
  else
    {
      ACE_OS::strcat (message, ACE_TEXT (" increased from "));
    }
  ACE_OS::strcat (message, lastStr);
  ACE_OS::strcat (message, ACE_TEXT (" to "));
  ACE_OS::strcat (message, nextStr);

  return ACE_OS::strdup (message);
}

int
main (int argc, char **argv)
{
  // Initialise the ORB instance.
  CORBA::ORB_var orbVar = CORBA::ORB_init (argc, argv, "");

  // Create the stock quote.
  StockQuote stockQuote (orbVar);

  // Connect to the event channel.
  stockQuote.connectToChannel ();

  // Send events.
  stockQuote.broadcast ();

  return 0;
}
