/*
 * Simple any push consumer.
 */

#include "AnyPushConsumerImpl.h"


AnyPushConsumerImpl::AnyPushConsumerImpl (CORBA::ORB_var orbVar, ChannelUtil *pUtil)
  : orbVar_ (orbVar), pUtil_ (pUtil), consumerVar_ (0)
{
  PortableServer::POA_var poaVar;
  PortableServer::ServantBase_var servantVar;
  PortableServer::ObjectId_var oidVar;
  CORBA::Object_var objVar;

  // Obtain a reference to the root POA and create the CORBA object.
  poaVar = pUtil_->getRootPoa ();
  servantVar = (PortableServer::ServantBase_var) this;
  oidVar = poaVar->activate_object (servantVar.in ());
  objVar = poaVar->id_to_reference (oidVar);
  consumerVar_ = CosNotifyComm::PushConsumer::_narrow (objVar.in ());

  // Connect to the channel.
  connectToChannel ();
}

AnyPushConsumerImpl::~AnyPushConsumerImpl ()
{
}

void
AnyPushConsumerImpl::disconnect_push_consumer ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnect callback invoked\n")));
}

void
AnyPushConsumerImpl::offer_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CosNotifyComm::InvalidEventType,CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Offer change callback invoked\n")));
}

void
AnyPushConsumerImpl::push (const CORBA::Any &event)
  ACE_THROW_SPEC ((CosEventComm::Disconnected,CORBA::SystemException))
{
  char message[1024];
  pUtil_->printEvent (message, event, 1);
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Received message: %s\n"), message));
}

void
AnyPushConsumerImpl::connectToChannel ()
{
  // Get a reference to the event channel.
  CosNotifyChannelAdmin::EventChannel_var channelVar = pUtil_->getChannel ();

  // Obtain a reference to the default consumer admin.
  CosNotifyChannelAdmin::ConsumerAdmin_var adminVar;
  ACE_TRY_NEW_ENV
    {
      adminVar = channelVar->default_consumer_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtained default consumer admin\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain default consumer admin\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain default consumer admin\n")));
      exit (1);
    }
  ACE_ENDTRY;

  CosNotifyChannelAdmin::ProxyID pid;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::ANY_EVENT;
  CosNotifyChannelAdmin::ProxySupplier_var psVar;

  // Obtain a proxy supplier.
  ACE_TRY_NEW_ENV
    {
      psVar = adminVar->obtain_notification_push_supplier (type, pid ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      if (CORBA::is_nil (psVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Obtained a null proxy supplier\n")));
          exit (1);
        }
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain a proxy supplier\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain a proxy supplier\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Narrow to a proxy push supplier.
  CosNotifyChannelAdmin::ProxyPushSupplier_var ppsVar = CosNotifyChannelAdmin::ProxyPushSupplier::_narrow (psVar.in ());
  if (CORBA::is_nil (ppsVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null proxy push supplier\n")));
      exit (1);
    }

  // Connect.
  ACE_TRY_NEW_ENV
    {
      ppsVar->connect_any_push_consumer (consumerVar_ ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected consumer to proxy\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to connect consumer to proxy\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to connect consumer to proxy\n")));
      exit (1);
    }
  ACE_ENDTRY;
}

int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  ChannelUtil *pUtil;
  AnyPushConsumerImpl *pConsumerImpl;

  // Initialise the ORB reference.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating ORB reference\n")));
  orbVar = CORBA::ORB_init (argc, argv);
  if (CORBA::is_nil (orbVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not create the ORB\n")));
      exit (1);
    }

  // Obtain a reference to the channel utility.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating channel utility\n")));
  pUtil = new ChannelUtil (orbVar);

  // Create the consumer object.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating consumer\n")));
  pConsumerImpl = new AnyPushConsumerImpl (orbVar, pUtil);

  // Enter the orb run loop.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Entering ORB run loop\n")));
  orbVar->run ();

  // Clean up and exit.
  delete pConsumerImpl;
  delete pUtil;
  return 0;
}
