// $Id: SSLExampleImpl.java,v 1.1 2007-03-15 10:18:45 tao Exp $

import java.io.*;
import org.omg.CORBA.*;
import org.omg.SecurityLevel2.*;
import org.omg.Security.*;


public class SSLExampleImpl extends SSLExamplePOA
{
   // General CORBA references.
   private org.omg.CORBA.ORB m_orb = null;
   private org.omg.PortableServer.POA m_rootPoa = null;
   private org.omg.PortableServer.POAManager m_rootPoaManager = null;
   private org.omg.SecurityLevel2.Current m_current = null;
   
   // ORB thread.
   public Thread m_orbThread = new Thread ("ORB")
   {
      public void run ()
      {
         m_orb.run ();
         System.out.println ("ORB thread ended");
      }
   };      

   // Boolean indicating if this is to run in automated test mode.
   private boolean m_test = false;
   
   // IOR file names for the three components.
   private String m_serverIORFile  = null;
   private String m_clientIORFile  = null;
   private String m_serviceIORFile = null;

   // CORBA references for the three components.
   private SSLExample m_client  = null;
   private SSLExample m_server  = null;
   private SSLExample m_service = null;
   
   // Booleans to indicate which component the subclass represents.
   private boolean m_isClient  = false;
   private boolean m_isServer  = false;
   private boolean m_isService = false;

   // Strings indicating which component to send data to.
   private String SERVER_STRING  = "server ";
   private String CLIENT_STRING  = "client ";
   private String SERVICE_STRING = "service ";
   
   // String with the "exit" command.
   private String EXIT_STRING = "exit";

   // References for the component's role.
   private String m_iorFile = null;
   private String m_roleName = null;
   
   
   public static void main (String[] args)
   {
      try
      {
         SSLExampleImpl impl = new SSLExampleImpl (args);

         if (impl.m_test)
         {
            System.out.println ("Running in automated test mode");
            impl.startTest ();
         }
         else
         {
            // Loop for the user input.
            impl.handleUserInput ();
         }
         
         // Shutdown the ORB and exit.
         System.out.println ("Shutting down ORB");
         impl.m_orb.shutdown (true);
         impl.delay (5000L);
      }
      catch (Throwable th)
      {
         System.err.println ("Error while running: " + th);
         th.printStackTrace ();
         System.exit (1);
      }

      System.exit (0);
   }

   
   SSLExampleImpl (String[] args)
      throws Exception
   {
      initialise (args);
      
      // Create the CORBA object for the client.
      org.omg.CORBA.Object obj = m_rootPoa.servant_to_reference (this);
      SSLExample example = SSLExampleHelper.narrow (obj);
      String ior = m_orb.object_to_string (example);
      System.out.println ("Activated " + m_roleName + " as " + ior);

      // Write out the IOR.
      writeIOR (m_iorFile, ior);
   }

   
   /**
    * Performs all general initialisation required.
    * @param args  The command line arguments
    */
   protected void initialise (String[] args)
      throws Exception
   {
      // Initialise the ORB and obtain/activate the root POA.
      m_orb = org.omg.CORBA.ORB.init (args, null);
      org.omg.CORBA.Object obj = m_orb.resolve_initial_references ("RootPOA");
      m_rootPoa = org.omg.PortableServer.POAHelper.narrow (obj);
      m_rootPoaManager = m_rootPoa.the_POAManager ();
      m_rootPoaManager.activate ();

      // Obtain a reference to the current environment.
      m_current = (org.omg.SecurityLevel2.Current) m_orb.resolve_initial_references ("SecurityCurrent");
      
      // Start the ORB thread.
      m_orbThread.start ();
      
      // Parse the command line arguments for the IOR file names and the role to play.
      m_serverIORFile  = extractArg (args, "-s", "server.ior");
      m_clientIORFile  = extractArg (args, "-c", "client.ior");
      m_serviceIORFile = extractArg (args, "-n", "service.ior");
      m_roleName       = extractArg (args, "-r", "<none>");
 
      // Parse the command line arguments for other options.
      m_test = isArgPresent (args, "-test");
      
      // Confirm that the role is correct and set the reference to the appropriate IOR file.
      if (m_roleName.equalsIgnoreCase ("server"))
      {
         m_isServer = true;
         m_iorFile  = m_serverIORFile;
      }
      else if (m_roleName.equalsIgnoreCase ("client"))
      {
         m_isClient = true;
         m_iorFile  = m_clientIORFile;
      }
      else if (m_roleName.equalsIgnoreCase ("service"))
      {
         m_isService = true;
         m_iorFile   = m_serviceIORFile;
      }
      else
      {
         System.err.println ("Role was not specified or was not recognised (must be either client, server or service)");
         System.exit (-1);
      }
   }
   

   /**
    * Checks the validity of a filename.
    * @param filename   The name of the file
    * @return boolean indicating the file is valid (true) or not (false)
    */
   protected boolean checkFile (String filename)
   {
      File f = new File (filename);
      if (! f.exists ())
      {
         System.out.println("File " + filename + " does not exist");
         return false;
      }
      if( f.isDirectory () )
      {
         System.out.println ("File " + filename + " is a directory");
         return false;
      }
      if (! f.canRead ())
      {
         System.out.println ("File " + filename + " cannot be read from");
         return false;
      }
      
      return true;
   }

   
   /**
    * Reads an IOR from a file.
    * @param filename  The name of the file containing the IOR
    * @return A string with the IOR
    */
   protected String readIOR (String filename)
      throws Exception
   {
      File f = new File (filename);
      BufferedReader br = new BufferedReader (new FileReader (f));
      String ior = br.readLine ();
      br.close ();
      
      return ior;
   }

   
   /**
    * Writes an IOR to a file.
    * @param filename  The name of the file to write to
    * @param ior       The IOR to write out
    */
   protected void writeIOR (String filename, String ior)
      throws Exception
   {
      BufferedWriter bw = new BufferedWriter (new FileWriter (new File (filename)));
      bw.write (ior);
      bw.flush ();
      bw.close ();
   }
   

   /**
    * Extracts a string value for a specific switch from an array of arguments.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @param def    The default value to use, if the argument is not found or
    *               if there is no argument following it.
    * @return       The found argument or the default.
    */
   protected synchronized String extractArg (String[] args, String arg, String def)
   {
      String ret = def;

      for (int i = 0 ; i < args.length ; i++)
      {
         if (args [i].equals (arg))
         {
            if (i <= (args.length-2))
            {
               ret = args [i+1];
            }
            break;
         }
      }

      return ret;
   }

   
   /**
    * Checks to see if a switch was present in an argument array.
    * @param arg   The switch to check for
    * @param args  The argument array to search
    * @return boolean  
    */
   private boolean isArgPresent (String[] args, String arg)
   {
      for (int i = 0 ; i < args.length ; i++)
      {
         if (args[i].equals (arg))
         {
            return true;
         }
      }
      
      return false;
   }   
   

   /**
    * Handles the user input; exits when the exit comment has been entered.
    */
   protected void handleUserInput ()
      throws Exception
   {
      System.out.println ("Ready: waiting for user input, \"" + EXIT_STRING + "\" to exit");
      if (m_isServer)
      {
         System.out.println (CLIENT_STRING + "xxxx      Send string xxxx to client");
      }
      if (m_isClient)
      {
         System.out.println (SERVER_STRING + "xxxx      Send string xxxx to server");
      }
      if (m_isServer || m_isClient)
      {
         System.out.println (SERVICE_STRING + "xxxx     Send string xxxx to service");
      }
        
      BufferedReader input = new BufferedReader (new InputStreamReader (System.in));
      String line = null;
      int status = 0;
      while (true)
      {
         line = input.readLine ();
         if (line == null || line.length () == 0)
         {
            continue;
         }

         try
         {
            status = parseLine (line);
         }
         catch (Throwable th)
         {
            System.err.println ("Failed to activate / send to target: " + th);
            th.printStackTrace ();
         }

         if (status == 1)
         {
            break;
         }
      }
   }

   
   /**
    * Parses a line of input / command.
    * @param line  String with the line to parse
    * @return Status: 1 if the user entered the exit command, 0 otherwise.
    */
   protected int parseLine (String line)
      throws Exception
   {
      // Note: we delay obtaining a reference to the other components until the first use,
      // so that all can be started and write out their IORs without trying to read the IORs
      // from the other two...

      if (line.equals (EXIT_STRING))
      {
         return 1;
      }
      
      if (line.startsWith (SERVER_STRING) && m_isClient)
      {
         String message = line.substring (SERVER_STRING.length ());
         
         if (m_server == null)
         {
            System.out.println ("Obtaining reference to server");
            checkFile (m_serverIORFile);
            String ior = readIOR (m_serverIORFile);
            org.omg.CORBA.Object obj = m_orb.string_to_object (ior);
            m_server = SSLExampleHelper.narrow (obj);
         }
         
         System.out.println ("Calling server with message: " + message);
         m_server.send_message (message);
      }
      else if (line.startsWith (CLIENT_STRING) && m_isServer)
      {
         String message = line.substring (CLIENT_STRING.length ());
         
         if (m_client == null)
         {
            System.out.println ("Obtaining reference to client");
            checkFile (m_clientIORFile);
            String ior = readIOR (m_clientIORFile);
            org.omg.CORBA.Object obj = m_orb.string_to_object (ior);
            m_client = SSLExampleHelper.narrow (obj);
         }
         
         System.out.println ("Calling client with message: " + message);
         m_client.send_message (message);
      }
      else if (line.startsWith (SERVICE_STRING) && ! m_isService)
      {
         String message = line.substring (SERVICE_STRING.length ());
         
         if (m_service == null)
         {
            System.out.println ("Obtaining reference to service");
            checkFile (m_serviceIORFile);
            String ior = readIOR (m_serviceIORFile);
            org.omg.CORBA.Object obj = m_orb.string_to_object (ior);
            m_service = SSLExampleHelper.narrow (obj);
         }
         
         System.out.println ("Calling service with message: " + message);
         m_service.send_message (message);
      }
      
      return 0;
   }
   

   /**
    * Delays execution.
    * @param duration  Time (in milliseconds)
    */
   protected void delay (long duration)
   {
      if (duration != 0)
      {
         try
         {
            Thread.currentThread ().sleep (duration);
         }
         catch (InterruptedException ie)
         {
            System.err.println ("Warning: delay interrupted");
         }
      }
   }


   public void send_message (String message)
   {
      System.out.print ("Received message: " + message);

      ReceivedCredentials creds = m_current.received_credentials ();
      if (creds != null)
      {
         System.out.println (" (invocation via SSLIOP)");
      }
      else
      {
         System.out.println (" (invocation via IIOP)");
      }
   }


   protected void startTest ()
     throws Exception
   {
      // Delay to allow all components to initialise.
      System.out.println ("Delaying before test start");
      delay (30000L);
      
      // Loop, sending to the other parts, with a small delay between iterations.
      for (int i = 1 ; i <= 10 ; i++)
      {
         String data = "Message #" + i + " from " + m_roleName;
         
         if (m_isServer)
         {
            parseLine (CLIENT_STRING + data);
            parseLine (SERVICE_STRING + data);
         }
         else if (m_isClient)
         {
            parseLine (SERVER_STRING + data);
            parseLine (SERVICE_STRING + data);
         }

         delay (1000L);
      }

      // Delay to allow any incoming calls to complete.
      System.out.println ("Delaying before test exit");
      delay (30000L);
   }
}
