// $Id: NodeManager_Impl.cpp,v 1.1.1.1 2007-06-22 14:15:07 sm Exp $

#include "ace/Log_Msg.h"
#include "ace/streams.h"

#include "NodeManager_Impl.h"
#include <ace/OS_Memory.h>

namespace DAnCE
  {
  NodeManager_Impl::NodeManager_Impl (CORBA::ORB_ptr orb
                                      , PortableServer::POA_ptr poa
                                      , const char* name
                                      , RedirectionService& redirection
                                      , CORBA::Boolean ignore_failures
                                      , TimeBase::TimeT ignore_failures_timeout)
      : orb_ (CORBA::ORB::_duplicate (orb))
      , poa_ (PortableServer::POA::_duplicate (poa))
      , name_ (name)
      , redirection_ (redirection)
      , ignore_failures_ (ignore_failures)
      , ignore_failures_timeout_ (ignore_failures_timeout)
  {
    redirection.add_node (name);
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::NodeManager_impl has been created\n"));
  }

  NodeManager_Impl::~NodeManager_Impl()
  {
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::~NodeManager_impl - started\n"));
    for (TManagers::iterator iter = this->managers_.begin();
         iter != this->managers_.end();
         ++iter)
      {
        PortableServer::ObjectId_var id = this->poa_->servant_to_id ((*iter).int_id_.in ());
        this->poa_->deactivate_object (id.in());
      }
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::~NodeManager_impl - finished\n"));
  }

  void
  NodeManager_Impl::joinDomain (
    const ::Deployment::Domain & /*theDomain*/,
    ::Deployment::TargetManager_ptr /*manager*/,
    ::Deployment::Logger_ptr /*log*/,
    ::CORBA::Long /*updateInterval*/
    ACE_ENV_ARG_DECL_WITH_DEFAULTS
  )
  ACE_THROW_SPEC ( (
                     ::CORBA::SystemException
                   ))
  {
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::joinDomain - started\n"));
    // Method will be implemented on next developemnt stage
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::joinDomain - finished\n"));
  }

  void
  NodeManager_Impl::leaveDomain (
    ACE_ENV_SINGLE_ARG_DECL_WITH_DEFAULTS
  )
  ACE_THROW_SPEC ( (
                     ::CORBA::SystemException
                   ))
  {
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::leaveDomain - started\n"));
    // Method will be implemented on next development stage
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::leaveDomain - finished\n"));
  }

  ::Deployment::NodeApplicationManager_ptr
  NodeManager_Impl::preparePlan (
    const ::Deployment::DeploymentPlan & plan,
    ::Deployment::ResourceCommitmentManager_ptr /*resourceCommitment*/
    ACE_ENV_ARG_DECL_WITH_DEFAULTS
  )
  ACE_THROW_SPEC ( (
                     ::CORBA::SystemException,
                     ::Deployment::StartError,
                     ::Deployment::PlanError
                   ))
  {
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::preparePlan - started for node %s and plan %s\n", this->name_.c_str(), plan.UUID.in()));
    // resourceCommitment will be used on next development stage
    if (0 == this->managers_.find (plan.UUID.in()))
      {
        // What should we do here if we already have application for this plan?
        // Probably it is mistake because we should previously call destroyApplication
        // before performe relaunching of application
        ACE_ERROR ( (LM_ERROR, "[%M] NodeManager_impl::preparePlan - ApplicationManager for the specified UUID already exists\n"));
        throw ::Deployment::PlanError();
      }

    ACE_DEBUG ( (LM_TRACE, "[%M] NodeManager_impl::preparePlan - creating NodeApplicationManager...\n"));
    NodeApplicationManager_Impl *tmp = 0;
    ACE_NEW_THROW_EX (tmp,
                      NodeApplicationManager_Impl (this->orb_.in(),
                                                   this->poa_.in(),
                                                   plan,
                                                   this->redirection_,
                                                   this->name_,
                                                   this->ignore_failures_,
                                                   this->ignore_failures_timeout_),
                      CORBA::NO_MEMORY());
    NodeApplicationManagerImpl_var manager = tmp;
    ACE_DEBUG ( (LM_TRACE, "[%M] NodeManager_impl::preparePlan - activating NodeApplicationManager...\n"));
    PortableServer::ObjectId_var id = this->poa_->activate_object (manager.in ());
    CORBA::Object_var nam = this->poa_->id_to_reference (id.in());
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::preparePlan - NodeApplicationManager created\n"));

    //there is an idea to check if plan.UUID really exists
    this->managers_.bind (plan.UUID.in(), manager);
    // narrow should return a nil reference if it fails.
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::preparePlan - finished\n"));
    return Deployment::NodeApplicationManager::_narrow (nam.in ());
  }

  void
  NodeManager_Impl::destroyManager (
    ::Deployment::NodeApplicationManager_ptr appManager
    ACE_ENV_ARG_DECL_WITH_DEFAULTS
  )
  ACE_THROW_SPEC ( (
                     ::CORBA::SystemException,
                     ::Deployment::StopError,
                     ::Deployment::InvalidReference
                   ))
  {
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::destroyManager - started\n"));

    for (TManagers::iterator iter = this->managers_.begin();
         iter != this->managers_.end();
         ++iter)
      {
        CORBA::Object_var obj = this->poa_->servant_to_reference ((*iter).int_id_.in ());
        if (appManager->_is_equivalent (obj.in ()))
          {
            PortableServer::ObjectId_var id = this->poa_->reference_to_id (obj.in ());
            this->poa_->deactivate_object (id.in());
            // Destructively rebind to an empty value
            NodeApplicationManagerImpl_var empty;
            this->managers_.rebind ((*iter).ext_id_, empty);
            this->managers_.unbind ((*iter).ext_id_);
            ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::destroyManager - finished\n"));
            return;
          }
      }

    ACE_ERROR ( (LM_ERROR, "[%M] NodeManager_impl::destroyManager - correponding NodeApplicationManager cannot be found\n"));
    throw ::Deployment::InvalidReference();
  }

  ::Deployment::Resources *
  NodeManager_Impl::getDynamicResources (
    ACE_ENV_SINGLE_ARG_DECL_WITH_DEFAULTS
  )
  ACE_THROW_SPEC ( (
                     ::CORBA::SystemException
                   ))
  {
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::getDynamicResources - started\n"));
    // Method will be implemented on next development stage
    ACE_DEBUG ( (LM_DEBUG, "[%M] NodeManager_impl::getDynamicResources - finished\n"));
    return 0;
  }
} //DAnCE

